<?php
/**
 * MicroEngine CMS
 * https://microengine.jp
 *
 * (c) 2018 MicroEngine Inc.
 */

namespace MicroEngine\Admin\Controllers;

use MicroEngine\Admin\Models\User;
use Psr\Http\Message\ResponseInterface;
use MicroEngine\Admin\Forms\UserCreateForm;
use MicroEngine\Admin\Forms\UserUpdateForm;
use MicroEngine\Core\Controllers\Controller;
use Psr\Http\Message\ServerRequestInterface;

class UserController extends Controller
{
    /**
     * @param  ServerRequestInterface $request
     * @param  ResponseInterface $response
     * @param  array $args
     * @return ResponseInterface
     */
    public function index(ServerRequestInterface $request, ResponseInterface $response, array $args): ResponseInterface
    {
        $this->c->userManager->required(['user', 'user_own']);

        $pageNo = $args['page_no'] ?? 1;
        $query = User::orderBy('id');

        $userCapability = $this->c->userManager->hasCapability('user');
        if (!$userCapability) {
            // userを持っていなければ、ログインユーザーのみに絞り込む
            $user = $this->c->auth->getUser();
            $query->where('id', $user->id);
        }
        $pagination = $query->pagination($pageNo);
        $pagination->setRouteInfo($this->c, 'admin.user.index');
        $roles = array_unnest(config('settings.roles', []), 'label');

        return $this->view->render(
            $response,
            'admin/views/user/index.twig',
            compact('pagination', 'roles', 'userCapability')
        );
    }

    /**
     * @param  ServerRequestInterface $request
     * @param  ResponseInterface $response
     * @param  array $args
     * @return ResponseInterface
     */
    public function create(ServerRequestInterface $request, ResponseInterface $response, array $args): ResponseInterface
    {
        $this->c->userManager->required('user');

        $form = new UserCreateForm();
        $roles = array_unnest(config('settings.roles', []), 'label');

        return $this->view->render(
            $response,
            'admin/views/user/create.twig',
            compact('form', 'roles')
        );
    }

    /**
     * @param  ServerRequestInterface $request
     * @param  ResponseInterface $response
     * @param  array $args
     * @return ResponseInterface
     */
    public function store(ServerRequestInterface $request, ResponseInterface $response, array $args): ResponseInterface
    {
        $this->c->userManager->required('user');

        $form = new UserCreateForm();
        $form->setInputValue($request->getParsedBody());

        // バリデーションチェック
        if ($form->validate()) {
            $user = $this->c->userManager->create($form->getInputValues());
            $this->flash->addMessage('success', "ユーザーを作成しました。ユーザー名：{$user->name} ( {$user->login_id} )");

            return $response->withRedirect($this->c->router->pathFor('admin.user.index'));
        } else {
            $roles = array_unnest(config('settings.roles', []), 'label');

            return $this->view->render(
                $response,
                'admin/views/user/create.twig',
                compact('form', 'roles')
            );
        }
    }

    /**
     * @param  ServerRequestInterface $request
     * @param  ResponseInterface $response
     * @param  array $args
     * @return ResponseInterface
     */
    public function edit(ServerRequestInterface $request, ResponseInterface $response, array $args): ResponseInterface
    {
        $this->c->userManager->required(['user', 'user_own']);
        $this->c->userManager->canEditOtherUser($args['id']);

        $form = new UserUpdateForm($args['id']);
        $user = User::findOrFail($args['id']);
        $form->setInputValue($user);
        $roles = array_unnest(config('settings.roles', []), 'label');

        return $this->view->render(
            $response,
            'admin/views/user/edit.twig',
            compact('form', 'user', 'roles')
        );
    }

    /**
     * @param  ServerRequestInterface $request
     * @param  ResponseInterface $response
     * @param  array $args
     * @return ResponseInterface
     */
    public function update(ServerRequestInterface $request, ResponseInterface $response, array $args): ResponseInterface
    {
        $this->c->userManager->required(['user', 'user_own']);
        $this->c->userManager->canEditOtherUser($args['id']);

        $form = new UserUpdateForm($args['id']);
        $form->setInputValue($request->getParsedBody());

        // バリデーションチェック
        if ($form->validate()) {
            $user = $this->c->userManager->update($args['id'], $form->getInputValues());
            $this->flash->addMessage('success', "ユーザーを更新しました。ユーザー名：{$user->name} ( {$user->login_id} )");

            return $response->withRedirect($this->c->router->pathFor('admin.user.index'));
        } else {
            $user = User::findOrFail($args['id']);
            $roles = array_unnest(config('settings.roles', []), 'label');

            return $this->view->render(
                $response,
                'admin/views/user/edit.twig',
                compact('form', 'user', 'roles')
            );
        }
    }

    /**
     * @param  ServerRequestInterface $request
     * @param  ResponseInterface $response
     * @param  array $args
     * @return ResponseInterface
     */
    public function unlock(ServerRequestInterface $request, ResponseInterface $response, array $args): ResponseInterface
    {
        $this->c->userManager->required('user');

        $user = $this->c->userManager->resetFailure($args['id']);
        $this->flash->addMessage('success', "ユーザーのロックを解除しました。ユーザー名：{$user->name} ( $user->login_id )");
        $this->c->logger->addNotice('ユーザーのロックアウトを解除しました。', [
            'login_id' => $user->login_id,
            'name' => $user->name,
            'role' => $user->role,
        ]);
        return $response->withRedirect($this->c->router->pathFor('admin.user.index'));
    }

    /**
     * @param  ServerRequestInterface $request
     * @param  ResponseInterface $response
     * @param  array $args
     * @return ResponseInterface
     */
    public function destroy(ServerRequestInterface $request, ResponseInterface $response, array $args): ResponseInterface
    {
        $this->c->userManager->required('user');
        $this->c->userManager->canEditOtherUser($args['id']);

        $user = $this->c->userManager->destroy($args['id']);
        $this->flash->addMessage('success', "ユーザーを削除しました。ユーザー名：{$user->name} ( {$user->login_id} )");

        return $response->withRedirect($this->c->router->pathFor('admin.user.index'));
    }
}
