<?php
/**
 * MicroEngine CMS
 * https://microengine.jp
 *
 * (c) 2018 MicroEngine Inc.
 */

namespace MicroEngine\Admin\Services;

use MicroEngine\Core\Support\FileSystem;
use Interop\Container\ContainerInterface;

class ConfigService
{
    /**
     * @var ContainerInterface
     */
    protected $c;
    /**
     * @var FileSystem
     */
    protected $fileSystem;

    /**
     * @param ContainerInterface $c
     */
    public function __construct(ContainerInterface $c)
    {
        $this->c = $c;
        $this->fileSystem = new FileSystem(system_root('config'));
    }

    /**
     * 設定ファイル名取得
     *
     * @param string $filename
     * @return string
     */
    public function getLabel(string $filename): string
    {
        $configs = config('settings.config_files');
        if (isset($configs[$filename])) {
            return $configs[$filename];
        }
        return '';
    }

    /**
     * 設定ファイルが編集可能か確認する
     *
     * @param string $filename
     * @return bool
     */
    public function isEditable(string $filename): bool
    {
        $configs = config('settings.config_files');
        if (isset($configs[$filename])) {
            return true;
        }
        return false;
    }

    /**
     * 設定ファイルの内容を取得
     *
     * @param string $path
     * @return string|bool
     * @throws \MicroEngine\Core\Exception\MeException
     */
    public function getContent(string $path)
    {
        return $this->fileSystem->get($path);
    }

    /**
     * 設定ファイルを保存
     *
     * @param string $path
     * @param string $content
     * @return bool|int
     * @throws \MicroEngine\Core\Exception\MeException
     */
    public function saveFile(string $path, string $content)
    {
        // バックアップファイルを作成する
        $this->fileSystem->copy($path, 'backup/' . $path);

        // キャッシュをクリアする
        $this->c->htmlCache->clear();

        // 新ファイルを保存する
        $result = $this->fileSystem->put($path, $content);

        $this->c->logger->addInfo('設定ファイルを保存しました。', [
            'path' => $path,
        ]);

        return $result;
    }

    /**
     * 設定更新
     * キーを指定して設定値を置換する
     *
     * @param string $path
     * @param array $config
     * @throws \MicroEngine\Core\Exception\MeException
     */
    public function update(string $path, array $config)
    {
        // 設定ファイルの内容を取得
        $content = $this->getContent($path);

        // 渡された設定のキーごとに値を置換する
        foreach ($config as $key => $value) {
            $escapedValue = str_replace("'", "\\'", $value);
            $content = preg_replace(
                "/'{$key}'\s*=>\s*'(.*?)(?<!\\\\)'/u",
                "'{$key}' => '{$escapedValue}'",
                $content);
        }
        // 更新したファイルを保存する
        $this->saveFile($path, $content);
    }
}
