<?php
/**
 * MicroEngine CMS
 * https://microengine.jp
 *
 * (c) 2018 MicroEngine Inc.
 */

namespace MicroEngine\Cms\Controllers;

use MicroEngine\Cms\Models\Page;
use MicroEngine\Cms\Models\Category;
use Psr\Http\Message\ResponseInterface;
use MicroEngine\Cms\Forms\CategoryForm;
use Interop\Container\ContainerInterface;
use MicroEngine\Core\Controllers\Controller;
use Psr\Http\Message\ServerRequestInterface;
use MicroEngine\Cms\Services\CategoryManager;

class CategoryController extends Controller
{
    /**
     * @var CategoryManager
     */
    protected $categoryManager;

    /**
     * @param ContainerInterface $c
     */
    public function __construct(ContainerInterface $c)
    {
        parent::__construct($c);

        $this->categoryManager = $c->categoryManager;
        // ライセンスチェック
        $this->c->license->hasLicense();
    }

    /**
     * @param  ServerRequestInterface $request
     * @param  ResponseInterface $response
     * @param  array $args
     * @return ResponseInterface
     */
    public function index(ServerRequestInterface $request, ResponseInterface $response, array $args): ResponseInterface
    {
        $categories = $this->categoryManager->sortCategoryTree(Category::all());

        return $this->view->render(
            $response,
            'cms/views/category/index.twig',
            compact('categories')
        );
    }

    /**
     * @param  ServerRequestInterface $request
     * @param  ResponseInterface $response
     * @param  array $args
     * @return ResponseInterface
     */
    public function create(ServerRequestInterface $request, ResponseInterface $response, array $args): ResponseInterface
    {
        $categories = $this->categoryManager->categoryOptions();
        $form = new CategoryForm();

        return $this->view->render(
            $response,
            'cms/views/category/create.twig',
            compact('categories', 'form')
        );
    }

    /**
     * @param  ServerRequestInterface $request
     * @param  ResponseInterface $response
     * @param  array $args
     * @return ResponseInterface
     */
    public function store(ServerRequestInterface $request, ResponseInterface $response, array $args): ResponseInterface
    {
        $categories = $this->categoryManager->categoryOptions();
        $form = new CategoryForm();
        $form->setInputValue($request->getParsedBody());

        // バリデーション
        if ($form->validate()) {
            $category = $this->categoryManager->create($form->getInputValues());
            $this->flash->addMessage('success', "カテゴリを作成しました。名前：{$category->name} ( {$category->dir} )");

            return $response->withRedirect($this->c->router->pathFor('admin.category.index'));
        } else {
            return $this->view->render(
                $response,
                'cms/views/category/create.twig',
                compact('categories', 'form')
            );
        }
    }

    /**
     * @param  ServerRequestInterface $request
     * @param  ResponseInterface $response
     * @param  array $args
     * @return ResponseInterface
     */
    public function show(ServerRequestInterface $request, ResponseInterface $response, array $args): ResponseInterface
    {
        history_set('page_info', $request);
        history_set('page_edit', $request);
        $id = $args['id'];
        $pageNo = $args['page_no'] ?? 1;
        $sortArg = $args['sort'] ?? 'updated_at';
        $sort = valid_parameter($sortArg, ['updated_at', 'name', 'path']);
        $orderArg = $args['order'] ?? 'desc';
        $order = valid_parameter($orderArg, ['desc', 'asc']);

        $category = Category::findOrFail($id);
        $pagesConfig = $this->categoryManager->filterPageByDir(config('pages'), $category->dir);
        $pagination = Page::notDeleted()->where('category_id', $id)->orderBy($sort, $order)->pagination($pageNo);
        $pagination->setRouteInfo($this->c, 'admin.category.show', ['id' => $id, 'sort' => $sort, 'order' => $order]);
        $sortOrder = $this->c->util->sortOrderClass($sort, $order, ['name', 'path', 'updated_at']);

        return $this->view->render(
            $response,
            'cms/views/category/show.twig',
            compact('category', 'pagesConfig', 'pagination', 'sortOrder')
        );
    }

    /**
     * @param  ServerRequestInterface $request
     * @param  ResponseInterface $response
     * @param  array $args
     * @return ResponseInterface
     */
    public function edit(ServerRequestInterface $request, ResponseInterface $response, array $args): ResponseInterface
    {
        $categories = $this->categoryManager->categoryOptions();
        $form = new CategoryForm($args['id']);
        $category = Category::findOrFail($args['id']);
        $form->setInputValue($category);

        return $this->view->render(
            $response,
            'cms/views/category/edit.twig',
            compact('categories', 'form')
        );
    }

    /**
     * @param  ServerRequestInterface $request
     * @param  ResponseInterface $response
     * @param  array $args
     * @return ResponseInterface
     */
    public function update(ServerRequestInterface $request, ResponseInterface $response, array $args): ResponseInterface
    {
        $categories = $this->categoryManager->categoryOptions();
        $form = new CategoryForm($args['id']);
        $form->setInputValue($request->getParsedBody());

        // バリデーション
        if ($form->validate()) {
            $category = $this->categoryManager->update($args['id'], $form->getInputValues());
            $this->flash->addMessage('success', "カテゴリを更新しました。名前：{$category->name} ( {$category->dir} )");

            return $response->withRedirect($this->c->router->pathFor('admin.category.index'));
        } else {
            return $this->view->render(
                $response,
                'cms/views/category/edit.twig',
                compact('categories', 'form')
            );
        }
    }

    /**
     * @param  ServerRequestInterface $request
     * @param  ResponseInterface $response
     * @param  array $args
     * @return ResponseInterface
     */
    public function destroy(ServerRequestInterface $request, ResponseInterface $response, array $args): ResponseInterface
    {
        $category = $this->categoryManager->destroy($args['id']);
        $this->flash->addMessage('success', "カテゴリを削除しました。カテゴリ：{$category->name} ( {$category->dir} )");

        return $response->withRedirect($this->c->router->pathFor('admin.category.index'));
    }

    /**
     * @param  ServerRequestInterface $request
     * @param  ResponseInterface $response
     * @param  array $args
     * @return ResponseInterface
     */
    public function modal(ServerRequestInterface $request, ResponseInterface $response, array $args): ResponseInterface
    {
        $categories = $this->categoryManager->sortCategoryTree(Category::all());

        return $this->view->render(
            $response,
            'cms/views/category/modal.twig',
            compact('categories')
        );
    }

    /**
     * @param  ServerRequestInterface $request
     * @param  ResponseInterface $response
     * @param  array $args
     * @return ResponseInterface
     */
    public function modalShow(ServerRequestInterface $request, ResponseInterface $response, array $args): ResponseInterface
    {
        $id = $args['id'];

        $category = Category::findOrFail($id);
        $parent = Category::find($category->parent_id);
        $children = Category::where('parent_id', $id)->get();
        $pagesConfig = config('pages');
        $pages = Page::notDeleted()->where('category_id', $id)->orderBy('display_order', 'asc')->get();

        return $this->view->render(
            $response,
            'cms/views/category/list.twig',
            compact('category', 'parent', 'children', 'pagesConfig', 'pages')
        );
    }
}
