<?php
/**
 * MicroEngine CMS
 * https://microengine.jp
 *
 * (c) 2018 MicroEngine Inc.
 */

namespace MicroEngine\Cms\Controllers;

use MicroEngine\Cms\Models\Collection;
use Psr\Http\Message\ResponseInterface;
use Interop\Container\ContainerInterface;
use MicroEngine\Cms\Forms\CollectionForm;
use Psr\Http\Message\ServerRequestInterface;
use MicroEngine\Core\Controllers\Controller;
use MicroEngine\Cms\Services\ContentService;
use MicroEngine\Cms\Services\CollectionManager;

class CollectionController extends Controller
{
    /**
     * @var CollectionManager
     */
    protected $collectionManager;

    /**
     * @param ContainerInterface $c
     */
    public function __construct(ContainerInterface $c)
    {
        parent::__construct($c);

        $this->collectionManager = $c->collectionManager;
        // ライセンスチェック
        $this->c->license->hasLicense();
    }

    /**
     * @param  ServerRequestInterface $request
     * @param  ResponseInterface $response
     * @param  array $args
     * @return ResponseInterface
     */
    public function index(ServerRequestInterface $request, ResponseInterface $response, array $args): ResponseInterface
    {
        $collections = config('collections');
        history_unset('collection');

        return $this->view->render(
            $response,
            'cms/views/collection/index.twig',
            compact('collections')
        );
    }

    /**
     * @param ServerRequestInterface $request
     * @param ResponseInterface $response
     * @param array $args
     * @return ResponseInterface
     * @throws \Exception
     */
    public function listing(ServerRequestInterface $request, ResponseInterface $response, array $args): ResponseInterface
    {
        history_set('collection', $request);

        $type = $args['type'];
        $pageNo = $args['page_no'] ?? 1;
        $sortArg = $args['sort'] ?? 'updated_at';
        $sort = valid_parameter($sortArg, ['updated_at', 'name']);
        $orderArg = $args['order'] ?? 'desc';
        $order = valid_parameter($orderArg, ['desc', 'asc']);
        $config = content_config($type, 'collection');
        $pagination = Collection::notDeleted()->where('type', $type)->orderBy($sort, $order)->pagination($pageNo);
        $pagination->setRouteInfo($this->c, 'admin.collection.listing', ['type' => $type, 'sort' => $sort, 'order' => $order]);
        $sortOrder = $this->c->util->sortOrderClass($sort, $order, ['name', 'updated_at']);

        return $this->view->render(
            $response,
            'cms/views/collection/listing.twig',
            compact('type', 'config', 'pagination', 'sortOrder')
        );
    }

    /**
     * @param ServerRequestInterface $request
     * @param ResponseInterface $response
     * @param array $args
     * @return ResponseInterface
     * @throws \Exception
     */
    public function create(ServerRequestInterface $request, ResponseInterface $response, array $args): ResponseInterface
    {
        $type = $args['type'];
        $config = content_config($type, 'collection');
        $form = new CollectionForm($this->c);
        $form->setConfig($config);
        $form->setInitialValues();
        $form->set('edit_mode', 'create');

        return $this->view->render(
            $response,
            'cms/views/collection/create.twig',
            compact('type', 'config', 'form')
        );
    }

    /**
     * @param ServerRequestInterface $request
     * @param ResponseInterface $response
     * @param array $args
     * @return ResponseInterface
     * @throws \Exception
     */
    public function copy(ServerRequestInterface $request, ResponseInterface $response, array $args): ResponseInterface
    {
        $collection = Collection::withoutGlobalScopes()->findOrFail($args['id']);
        $type = $collection->type;
        $config = content_config($type, 'collection');
        unset($collection->id);
        $form = new CollectionForm($this->c);
        $form->setConfig($config);
        $form->setInputValue($collection);
        $form->set('edit_mode', 'copy');

        return $this->view->render(
            $response,
            'cms/views/collection/create.twig',
            compact('type', 'config', 'form')
        );
    }

    /**
     * @param ServerRequestInterface $request
     * @param ResponseInterface $response
     * @param array $args
     * @return ResponseInterface
     * @throws \Exception
     */
    public function store(ServerRequestInterface $request, ResponseInterface $response, array $args): ResponseInterface
    {
        $type = $args['type'];
        $config = content_config($type, 'collection');
        $form = new CollectionForm($this->c);
        $form->setConfig($config);
        $form->setInputValue($request->getParsedBody());

        // バリデーション
        if ($form->validate()) {
            $values = $form->getInputValues();
            $collection = $this->collectionManager->create($values);
            if ($form->get('revision_type') === 'draft') {
                $this->flash->addMessage('success', 'コレクションを下書きとして保存しました。コレクション：' . $collection->name);
            } else {
                $this->flash->addMessage('success', 'コレクションを作成しました。コレクション：' . $collection->name);
            }

            if ($form->get('edit_mode') === 'copy') {
                // 複製した場合はhistoryに関係なく新コンテンツの情報画面に遷移させる。
                return $response->withRedirect($this->c->router->pathFor('admin.collection.info', ['id' => $collection->id]));
            }
            return $response->withRedirect($this->c->router->pathFor('admin.collection.listing', ['type' => $collection->type]));
        } else {
            $mode = $request->getQueryParam('mode');

            return $this->view->render(
                $response,
                'cms/views/collection/create.twig',
                compact('type', 'config', 'form', 'mode')
            );
        }
    }

    /**
     * @param ServerRequestInterface $request
     * @param ResponseInterface $response
     * @param array $args
     * @return ResponseInterface
     * @throws \Exception
     */
    public function preview(ServerRequestInterface $request, ResponseInterface $response, array $args): ResponseInterface
    {
        $type = $args['type'];
        $config = content_config($type, 'collection');
        $form = new CollectionForm($this->c);
        $form->setConfig($config);
        $form->setInputValue($request->getParsedBody());
        $site = $this->c->site;
        $site->setPreview(true);

        // タイプ別プレビューテンプレートのパス
        $template = '@site/collection/preview/' . $type . '.twig';
        if (!$this->view->getLoader()->exists($template)) {
            // タイプ別のテンプレートが用意されていない場合は汎用テンプレートを指定する。
            $template = 'cms/views/base/preview.twig';
        }

        // バリデーション
        if ($form->validate()) {
            $values = $form->getInputValues();
            $collection = new Collection($values);
            $site->pushContent($collection, true);

            return $this->view->render(
                $response,
                $template,
                compact('collection', 'config')
            );
        } else {
            return $this->view->render(
                $response,
                'cms/views/base/previewError.twig',
                compact('form')
            );
        }
    }

    /**
     * @param ServerRequestInterface $request
     * @param ResponseInterface $response
     * @param array $args
     * @return ResponseInterface
     * @throws \Exception
     */
    public function edit(ServerRequestInterface $request, ResponseInterface $response, array $args): ResponseInterface
    {
        $id = $args['id'] ?? null;
        $editMode = $args['edit_mode'] ?? 'edit';
        $revisionId = $args['revision_id'] ?? null;
        if ($revisionId) {
            $collection = $this->collectionManager->revision($id, $revisionId);
        } else {
            $collection = Collection::withoutGlobalScopes()->findOrFail($id);
        }
        // 公開設定が-5の場合は、ごみ箱に入ったコンテンツなので、編集モードを trash にする。
        if ($collection->visibility == ContentService::VISIBILITY_TRASH) {
            $editMode = 'trash';
            // ごみ箱内のコンテンツは公開をセットする
            $collection->visibility = ContentService::VISIBILITY_PUBLIC;
        }
        $type = $collection->type;
        $config = content_config($type, 'collection');
        $form = new CollectionForm($this->c);
        $form->setId($args['id']);
        $form->setConfig($config);
        $form->setInputValue($collection);
        $form->set('edit_mode', $editMode);
        $form->set('revision_id', $revisionId);

        return $this->view->render(
            $response,
            'cms/views/collection/edit.twig',
            compact('type', 'config', 'form')
        );
    }

    /**
     * @param ServerRequestInterface $request
     * @param ResponseInterface $response
     * @param array $args
     * @return ResponseInterface
     * @throws \Exception
     */
    public function update(ServerRequestInterface $request, ResponseInterface $response, array $args): ResponseInterface
    {
        $collection = Collection::withoutGlobalScopes()->findOrFail($args['id']);
        $type = $collection->type;
        $config = content_config($type, 'collection');
        $form = new CollectionForm($this->c);
        $form->setId($args['id']);
        $form->setConfig($config);
        $form->setInputValue($request->getParsedBody());

        // バリデーション
        if ($form->validate()) {
            $values = $form->getInputValues();
            $collection = $this->collectionManager->update($args['id'], $values);
            if ($form->get('revision_type') === 'draft') {
                $this->flash->addMessage('success', 'コレクションを下書きとして保存しました。名前：' . $collection->name);
            } else {
                $this->flash->addMessage('success', 'コレクションを更新しました。名前：' . $collection->name);
            }

            if (history_has('collection')) {
                return $response->withRedirect(history('collection'));
            }
            return $response->withRedirect($this->c->router->pathFor('admin.collection.listing', ['type' => $collection->type]));
        } else {
            return $this->view->render(
                $response,
                'cms/views/collection/edit.twig',
                compact('type', 'config', 'form')
            );
        }
    }

    /**
     * @param ServerRequestInterface $request
     * @param ResponseInterface $response
     * @param array $args
     * @return ResponseInterface
     * @throws \Exception
     */
    public function info(ServerRequestInterface $request, ResponseInterface $response, array $args): ResponseInterface
    {
        history_set('collection', $request);

        $collection = Collection::withoutGlobalScopes()->findOrFail($args['id']);
        $type = $collection->type;
        $config = content_config($type, 'collection');
        $revisions = $this->collectionManager->revisionList($collection);

        return $this->view->render(
            $response,
            'cms/views/collection/info.twig',
            compact('collection', 'type', 'config', 'revisions')
        );
    }

    /**
     * @param  ServerRequestInterface $request
     * @param  ResponseInterface $response
     * @param  array $args
     * @return ResponseInterface
     */
    public function destroy(ServerRequestInterface $request, ResponseInterface $response, array $args): ResponseInterface
    {
        $id = $args['id'];
        $editMode = $args['edit_mode'] ?? null;
        $revisionId = $args['revision_id'] ?? null;

        $collection = $this->collectionManager->destroy($id, $revisionId);
        if ($editMode === 'revision') {
            $message = '履歴データを削除しました。';
        } elseif ($editMode === 'draft') {
            $message = '下書きを削除しました。';
        } else {
            $message = 'コレクションをごみ箱に移動しました。';
        }
        $this->flash->addMessage('success', $message . '名前：' . $collection->name);

        if ($editMode !== null && history_has('collection')) {
            return $response->withRedirect(history('collection'));
        }
        return $response->withRedirect($this->c->router->pathFor('admin.collection.listing', ['type' => $collection->type]));
    }

    /**
     * @param  ServerRequestInterface $request
     * @param  ResponseInterface $response
     * @param  array $args
     * @return ResponseInterface
     */
    public function trash(ServerRequestInterface $request, ResponseInterface $response, array $args): ResponseInterface
    {
        $collectionsConfig = config('collections');
        // ごみ箱内のコンテンツを取得
        $collections = Collection::trashed()->orderBy('updated_at', 'desc')->get();

        return $this->view->render(
            $response,
            'cms/views/collection/trash.twig',
            compact('collections', 'collectionsConfig')
        );
    }

    /**
     * @param  ServerRequestInterface $request
     * @param  ResponseInterface $response
     * @param  array $args
     * @return ResponseInterface
     */
    public function forceDestroy(ServerRequestInterface $request, ResponseInterface $response, array $args): ResponseInterface
    {
        $id = $args['id'];

        $collection = $this->collectionManager->forceDestroy($id);
        $this->flash->addMessage('success', 'コレクションをごみ箱から削除しました。名前：' . $collection->name);

        return $response->withRedirect($this->c->router->pathFor('admin.collection.trash'));
    }

    /**
     * @param  ServerRequestInterface $request
     * @param  ResponseInterface $response
     * @param  array $args
     * @return ResponseInterface
     */
    public function emptyTrash(ServerRequestInterface $request, ResponseInterface $response, array $args): ResponseInterface
    {
        // ごみ箱を空にして、削除したコンテンツ数を取得
        $count = $this->collectionManager->emptyTrash();
        $this->flash->addMessage('success', 'ごみ箱を空にしました。削除コレクション数：' . $count);

        return $response->withRedirect($this->c->router->pathFor('admin.collection.trash'));
    }
}
