<?php
/**
 * MicroEngine CMS
 * https://microengine.jp
 *
 * (c) 2018 MicroEngine Inc.
 */

namespace MicroEngine\Cms\Managers;

use Illuminate\Database\Eloquent\Model;

trait RevisionTrait
{
    /**
     * 指定ページの更新履歴一覧を取得
     *
     * @param Model $model
     * @return mixed
     */
    public function revisionList(Model $model)
    {
        return $this->revisionClass::where($this->modelId, $model->id)->orderBy('updated_at', 'desc')
            ->with('updateUser')->get();
    }

    /**
     * 指定したリビジョンIDのページオブジェクトを取得
     *
     * @param $id
     * @param $revisionId
     * @return Model
     */
    public function revision(int $id, int $revisionId): Model
    {
        // ページとページリビジョンオブジェクトを取得
        $model = $this->modelClass::withoutGlobalScopes()->findOrFail($id);
        $revision = $this->revisionClass::findOrFail($revisionId);

        // ページリビジョンの属性を取得
        $attributes = $revision->getAttributes();

        // モデルの casts プロパティを展開して array が指定されているものは json を連想配列に変換する
        foreach ($model->getCasts() as $field => $cast) {
            if ($cast === 'array') {
                $attributes[$field] = json_decode($attributes[$field], true);
            }
        }
        // fillを使ってリビジョン側の属性をセットする。
        $model->fill($attributes);

        return $model;
    }

    /**
     * 更新履歴を保存する
     *
     * @param int $id
     * @param array $attributes
     */
    protected function saveRevision(int $id, array $attributes)
    {
        $revisionType = $attributes['revision_type'] ?? 'trunk';
        if ($revisionType === 'draft') {
            // 更新履歴保存数を確認
            $revisionMaxCount = config('cms.draft_max', 1);
            if ($revisionMaxCount < 1) {
                // 下書きはリビジョンとして保存するので最低1件の履歴は残す
                $revisionMaxCount = 1;
            }
        } else {
            // 更新履歴保存数を確認
            $revisionMaxCount = config('cms.revision_max', 0);
        }

        // 1より小さい数値の場合は更新履歴は保存しない
        if (!is_integer($revisionMaxCount) || $revisionMaxCount < 1) {
            $this->destroyRevision($id, $revisionType, 0);
            return;
        }

        // ページIDをセットする
        $attributes[$this->modelId] = $id;
        // idプロパティは削除する。
        unset($attributes['id']);
        $revision = $this->revisionClass::create($attributes);
        // 古い履歴を削除する
        $this->destroyRevision($id, $revisionType, $revisionMaxCount);

        return $revision;
    }

    /**
     * ID指定で古い更新履歴を削除する
     *
     * @param int $id
     * @param string $revisionType
     * @param int $revisionMaxCount
     * @return mixed
     */
    protected function destroyRevision(int $id, string $revisionType = 'trunk', int $revisionMaxCount = 0)
    {
        // 履歴の最大数を越えるレコードのidを取得
        $deleteIds = $this->revisionClass::where($this->modelId, $id)
            ->where('revision_type', $revisionType)
            ->orderBy('updated_at', 'desc')
            ->offset($revisionMaxCount)
            ->limit(PHP_INT_MAX)
            ->pluck('id')
            ->toArray();
        // 取得したidをまとめて削除する
        return $this->revisionClass::destroy($deleteIds);
    }
}
