<?php
/**
 * MicroEngine CMS
 * https://microengine.jp
 *
 * (c) 2018 MicroEngine Inc.
 */

namespace MicroEngine\Cms\Services;

use Slim\Views\Twig;
use MicroEngine\Cms\Models\Page;
use MicroEngine\Cms\Models\Table;
use MicroEngine\Cms\Models\Module;
use MicroEngine\Cms\Models\Calendar;
use MicroEngine\Cms\Models\Collection;
use Illuminate\Database\Eloquent\Model;
use Interop\Container\ContainerInterface;

class ContentService
{
    /**
     * @var int 公開設定：公開
     */
    const VISIBILITY_PUBLIC = 1;
    /**
     * @var int 公開設定：非公開
     */
    const VISIBILITY_PRIVATE = 0;
    /**
     * @var int 公開設定：下書き
     */
    const VISIBILITY_DRAFT = -1;
    /**
     * @var int 公開設定：ごみ箱
     */
    const VISIBILITY_TRASH = -5;
    /**
     * @var ContainerInterface
     */
    protected $c;

    /**
     * @param ContainerInterface $c
     */
    public function __construct(ContainerInterface $c)
    {
        $this->c = $c;
    }

    /**
     * 入力テンプレートパスを取得
     *
     * @param string $fieldName
     * @param array $config
     * @return string
     */
    public function inputTemplatePath(string $fieldName, array $config): string
    {
        /* @var Twig $view */
        $view = $this->c->view;

        // site にフィールド設定をセットする
        $this->c->site->setFieldConfig($config);

        $template = $config['input_template'] ?? $config['type'] ?? $fieldName;
        if (!$template) {
            // テンプレートが無い場合は空文字を返す
            return '';
        }

        // 公開テーマ内で入力テンプレートがないか確認する
        $templatePath = '@site/inputs/' . $template . '.twig';
        if ($view->getLoader()->exists($templatePath)) {
            return $templatePath;
        }

        // Cmsパッケージ内の入力テンプレートを確認する
        $templatePath = 'cms/views/inputs/' . $template . '.twig';
        if ($view->getLoader()->exists($templatePath)) {
            return $templatePath;
        } else {
            return '';
        }
    }

    /**
     * モジュール設定に該当するモデルをセットして返す
     *
     * @return array
     */
    public function moduleConfig(): array
    {
        // モジュール設定を取得
        $config = config('modules');
        // すべてのモジュールのレコードをDBから取得
        $rows = Module::all();
        foreach ($rows as $row) {
            if (isset($config[$row->type])) {
                // 設定のタイプに一致するDBレコードをセットする
                $config[$row->type]['content'] = $row;
            }
        }
        return $config;
    }

    /**
     * テーブル設定に該当するモデルをセットして返す
     *
     * @return array
     */
    public function tableConfig(): array
    {
        // テーブル設定を取得
        $config = config('tables');
        // 全てのテーブルコンテンツのレコードをDBから取得
        $rows = Table::all();
        foreach ($rows as $row) {
            if (isset($config[$row->type])) {
                // 設定のタイプに一致するDBレコードをセットする
                $config[$row->type]['content'] = $row;
            }
        }
        return $config;
    }

    /**
     * カスタムフィールドをメタテーブルに保存する
     *
     * @param Model $content
     * @throws \Exception
     */
    public function saveMeta(Model $content)
    {
        // カスタムフィールド設定を取得
        $contentConfig = content_config($content->type, $content->getBaseType());

        // メタ設定が無い場合は、保存しない。
        if (empty($contentConfig) || !isset($contentConfig['meta'])) {
            return;
        }

        // 新メタデータを集める
        $newMetas = [];
        foreach ($contentConfig['meta'] as $name => $fieldDotInput) {
            $values = array_get($content->fields, $fieldDotInput);
            if (is_array($values)) {
                foreach ($values as $value) {
                    // 値が配列の場合は、要素毎に分解する
                    $newMetas[] = [
                        'name' => $name,
                        'value' => $value,
                    ];
                }
            } else if ($values !== null) {
                $newMetas[] = [
                    'name' => $name,
                    'value' => $values,
                ];
            }
        }

        // DB保存済みメタデータを取得
        $oldMetas = $content->metas()->get();
        // 新旧でname,valueとも一致する場合はなにもしないので除外する
        foreach ($oldMetas as $oldIndex => $oldMeta) {
            foreach ($newMetas as $newIndex => $newMeta) {
                if ($oldMeta->name === $newMeta['name'] && $oldMeta->value === $newMeta['value']) {
                    // name, value とも一致する場合は変更不要なので両方とも除外する
                    unset($oldMetas[$oldIndex], $newMetas[$newIndex]);
                }
            }
        }

        // 旧レコードにnameが一致するものがあれば更新する。なければ新規登録する。
        foreach ($newMetas as $newIndex => $newMeta) {
            foreach ($oldMetas as $oldIndex => $oldMeta) {
                if ($oldMeta->name === $newMeta['name']) {
                    // nameが一致すればそのレコードを更新する
                    $oldMeta->value = $newMeta['value'];
                    $content->metas()->save($oldMeta);
                    // 処理したoldMetaは除外する
                    unset($oldMetas[$oldIndex]);
                    // 新規インサートを避けるために、外側のforeachの先頭に戻る
                    continue 2;
                }
            }
            // 一致するものが無ければ新規登録する
            $content->metas()->create($newMeta);
        }

        // 残った旧レコードを削除する
        foreach ($oldMetas as $oldMeta) {
            $oldMeta->delete();
        }
    }

    /**
     * コンテンツの状態が変わる最も近い日時のunixtimeを取得
     *
     * @return int|string
     */
    public function closestChangeUnixtime()
    {
        $closestChangeUnixtime = '';
        $timestamps = [];

        // 各コンテンツ毎に公開開始日と公開終了日の近い日時を取得
        $timestamps[] = Page::closestPublishedAt()->min('published_at');
        $timestamps[] = Page::closestClosedAt()->min('closed_at');
        $timestamps[] = Collection::closestPublishedAt()->min('published_at');
        $timestamps[] = Collection::closestClosedAt()->min('closed_at');
        $timestamps[] = Calendar::closestPublishedAt()->min('published_at');
        $timestamps[] = Calendar::closestClosedAt()->min('closed_at');
        // null を除外する
        $timestampsNotNull = array_filter($timestamps);
        if (count($timestampsNotNull)) {
            // 最も近い（小さい）日付を取得
            $closestChangeAt = min($timestampsNotNull) ?? '';
            if (!empty($closestChangeAt)) {
                // 日付文字列をunixtimeに変換する
                $closestChangeUnixtime = strtotime($closestChangeAt);
            }
        }

        return $closestChangeUnixtime;
    }

    /**
     * カスタムフィールドを検索テーブルに保存する
     *
     * @param Model $content
     * @throws \Exception
     */
    public function saveSearch(Model $content)
    {
        // カスタムフィールド設定を取得
        $contentConfig = content_config($content->type, $content->getBaseType());

        // search設定が無い場合は、保存しない。
        $searchConfig = $contentConfig['search'] ?? false;
        if (!$searchConfig) {
            return;
        }

        // 新検索データを集める
        $newSearch = [];
        // コンテンツ名は検索データに含める
        $newSearch[] = $content->name;
        // フィールドリストを取得
        $fields = $contentConfig['fields'] ?? [];
        // コンテンツ設定のフィールドリストごとに処理する
        foreach ($fields as $fieldName => $fieldConfig) {
            // フィールドの値を取り出す
            $values = $content->fields[$fieldName];
            // コンテナフィールドかどうかを確認
            $container = $fieldConfig['container'] ?? false;
            if ($container) {
                // コンテナであれば、そのコンテナ内のブロックの値を取り出す
                foreach ($values as $blockValues) {
                    // ブロックフィールドな名前と値に展開する
                    foreach ($blockValues as $blockFieldName => $blockFieldValue) {
                        // フィールド設定から該当ブロックの設定を取り出す
                        $blockFieldConfig = $fieldConfig['fields'][$blockFieldName];
                        $this->retrieveText($newSearch, $blockFieldConfig, $blockFieldValue);
                    }
                }
            } else {
                $this->retrieveText($newSearch, $fieldConfig, $values);
            }
        }
        // タグを取り除く
        $newSearch = array_map('strip_tags', $newSearch);
        // 連結する
        $search = implode('|', $newSearch);

        // DB保存済み検索データを取得
        $oldSearch = $content->search;
        if ($oldSearch) {
            // 既存データがあれば、新しい値をセットして保存
            $oldSearch->content = $search;
            $content->search()->save($oldSearch);
        } else {
            // 既存データながければ新規保存
            $content->search()->create(['content' => $search]);
        }
    }

    /**
     * フィールドからテキストを取り出す
     *
     * @param array &$search
     * @param array $fieldConfig
     * @param array $values
     */
    protected function retrieveText(array &$search, array $fieldConfig, array $values)
    {
        foreach ($fieldConfig['inputs'] as $inputName => $inputConfig) {
            if (isset($inputConfig['search']) && $inputConfig['search'] === 0) {
                // search プロパティに 0 が設定されている場合は検索データには含めない
                continue;
            }
            if ($this->isSearchValue($values[$inputName])) {
                // 検索データに含める値であることを確認してから追加する
                $search[] = $values[$inputName];
            }
        }
    }

    /**
     * 検索テーブルに登録するデータかどうかを判定する
     *
     * @param mixed $value
     * @return bool
     */
    protected function isSearchValue($value): bool
    {
        $targets = ['integer', 'double', 'string'];
        $type = gettype($value);
        if (!in_array($type, $targets, true)) {
            // $targets で指定した型に一致しない場合はスルー
            return false;
        }
        if ($type === 'string' && $value === '') {
            // 空文字の場合はスルー
            return false;
        }
        return true;
    }
}
