<?php
/**
 * MicroEngine CMS
 * https://microengine.jp
 *
 * (c) 2018 MicroEngine Inc.
 */

namespace MicroEngine\Cms\Services;

use Intervention\Image\Image;
use Intervention\Image\ImageManager;
use MicroEngine\Core\Support\FileSystem;
use Interop\Container\ContainerInterface;

class ImageUtility
{
    /**
     * @var ContainerInterface
     */
    protected $c;
    /**
     * @var FileSystem
     */
    protected $fileSystem;
    /**
     * @var array
     */
    protected $config;

    /**
     * @param ContainerInterface $c
     */
    public function __construct(ContainerInterface $c)
    {
        $this->c = $c;
        $this->config = config('image');
        $this->fileSystem = new FileSystem(document_root('files'));
    }

    /**
     * リクエストされたタイミングでリサイズ画像を用意する
     *
     * @param string $path
     * @return bool|Image
     * @throws \MicroEngine\Core\Exception\MeException
     */
    public function onTheFly(string $path)
    {
        // 拡張子確認
        $pathInfo = pathinfo($path);
        $filename = $pathInfo['filename'];
        $extension = $pathInfo['extension'];

        // 対応拡張子の確認
        if (!in_array(strtolower($extension), $this->config['resize'])) {
            // 対応拡張子以外は処理しない
            return false;
        }

        // 対応ファイル名の確認
        $pieces = explode('_', $filename);
        if (count($pieces) < 2) {
            // _ が含まれていなければ処理しない。
            return false;
        }
        // サフィックスを取り出す
        $suffix = array_pop($pieces);

        // オプションに一致することを確認する
        if (!isset($this->config['options'][$suffix])) {
            // オプションに一致しない場合は、処理しない。
            return false;
        }
        $option = $this->config['options'][$suffix];
        $original = $pathInfo['dirname'] . '/' . implode('_', $pieces) . '.' . $extension;

        // オリジナルファイルの有無確認
        if (!$this->fileSystem->has($original)) {
            return false;
        }
        $fullPath = $this->fileSystem->fullPath($original);
        // リサイズを実行する
        $newImage = $this->processing($fullPath, $option);

        if ($newImage) {
            // ファイルを保存する。
            $this->fileSystem->put($path, $newImage->encode());
        }
        return $newImage;
    }

    /**
     * 画像加工処理
     *
     * @param string $path
     * @param array $option
     * @return Image
     */
    public function processing(string $path, array $option): Image
    {
        $driver = $this->config['driver'] ?? 'gd';
        $manager = new ImageManager(['driver' => $driver]);

        $image = $manager->make($path);

        $mode = $option['mode'] ?? 'resize';
        switch ($mode) {
            case 'crop':
                $this->crop($image, $option['width'], $option['height']);
                break;
            case 'fill':
                $this->fill($image, $option['width'], $option['height']);
                break;
            case 'stretch':
                $this->stretch($image, $option['width'], $option['height']);
                break;
            case 'resize':
            default:
                $this->resize($image, $option['width'], $option['height']);
                break;
        }

        return $image;
    }

    /**
     * Resize
     * アスペクト比固定でリサイズする。
     * アスペクト比が異なる場合は、指定値より小さくなる。
     *
     * @param Image $image
     * @param int $width
     * @param int $height
     */
    protected function resize(Image $image, int $width, int $height)
    {
        $image->resize($width, $height, function ($constraint) {
            // アスペクト比固定
            $constraint->aspectRatio();
            // 拡大しない
            $constraint->upsize();
        });
    }

    /**
     * Crop
     * 指定値にリサイズする。
     * アスペクト比が合わない部分は、切り取る。
     *
     * @param Image $image
     * @param int $width
     * @param int $height
     */
    protected function crop(Image $image, int $width, int $height)
    {
        $image->fit($width, $height);
    }

    /**
     * Fill
     * 指定値にリサイズする。
     * アスペクト比が合わない部分は、背景色で埋めて合わせる。
     *
     * @param Image $image
     * @param int $width
     * @param int $height
     */
    protected function fill(Image $image, int $width, int $height)
    {
        $image->resize($width, $height, function ($constraint) {
            // アスペクト比固定
            $constraint->aspectRatio();
            // 拡大しない
            $constraint->upsize();
        });
        $image->resizeCanvas($width, $height, 'center', false, [255, 255, 255, 0]);
    }

    /**
     * Stretch
     * 指定値にリサイズする。
     * アスペクト比が合わない部分は、引き伸ばして合わせる。
     *
     * @param Image $image
     * @param int $width
     * @param int $height
     */
    protected function stretch(Image $image, int $width, int $height)
    {
        $image->resize($width, $height);
    }
}
