<?php
/**
 * MicroEngine CMS
 * https://microengine.jp
 *
 * (c) 2018 MicroEngine Inc.
 */

if (!function_exists('theme')) {
    /**
     * 公開サイトテーマのサイトルートパスを返します
     * 例： /themes/{テーマ名}
     *
     * @param string $path 配下のパス
     * @return string
     */
    function theme(string $path = ''): string
    {
        $theme = config('cms.theme', 'sample');

        return themes($theme) . ($path ? '/' . $path : $path);
    }
}

if (!function_exists('theme_root')) {
    /**
     * theme ディレクトリのファイルシステムパスを返す
     * 例： /filesystem/document_root/themes/{テーマ名}
     *
     * @param string $path 配下のパス
     * @return string
     */
    function theme_root(string $path = ''): string
    {
        return document_root() . theme() . ($path ? '/' . $path : $path);
    }
}

if (!function_exists('content_config')) {
    /**
     * フィールド設定を含むコンテンツ設定を取得する
     *
     * @param string $type
     * @param string $baseType
     * @return array
     * @throws Exception
     */
    function content_config(string $type, string $baseType): array
    {
        $config = config($baseType . 's');
        $fieldsConfig = fields_config();

        // 指定されたコンテンツの設定があることを確認する
        if (!isset($config[$type])) {
            throw new \MicroEngine\Core\Exception\MeException('コンテンツ設定が見つかりません。');
        }
        $config = $config[$type];

        // fields を持たない場合はフィールド設定を取得する必要がないので $config を返す
        if (empty($config['fields'])) {
            return $config;
        }

        // フィールド設定をコンテンツ設定の fields に統合する
        foreach ($config['fields'] as $fieldName => &$field) {
            if (isset($fieldsConfig[$field['type']])) {
                // フィールド設定があればコンテンツ側で定義されたフィールドの初期値とする
                $field = array_replace_recursive($fieldsConfig[$field['type']], $field);
            }
            // コンテナフィールドの場合は子フィールドの情報をセットする
            if (isset($field['fields'])) {
                $childFields = [];
                foreach ($field['fields'] as $childFieldType) {
                    // コンテナフィールドのフィールドリストに設定を追加する
                    $childFields[$childFieldType] = $fieldsConfig[$childFieldType];
                }
                // コンテナフィールドの fields プロパティを設定が定義された状態に置き換える。
                $field['fields'] = $childFields;
            }
        }
        return $config;
    }
}

if (!function_exists('fields_config')) {
    /**
     * フィールド設定を取得する
     *
     * @return array
     */
    function fields_config(): array
    {
        // fields.php 設定ファイルを読み込む
        $fieldsConfig = config('fields');

        foreach ($fieldsConfig as &$field) {
            if (isset($field['attr']) && $field['attr']) {
                // フィールド設定で attr プロパティが有効な場合は、属性用インプット設定を追加
                $field['inputs'] += [
                    'attr_id' => [
                        'label' => 'ID',
                        'rule' => '',
                        'value' => '',
                    ],
                    'attr_class' => [
                        'label' => 'クラス',
                        'rule' => '',
                        'value' => '',
                    ],
                    'attr_style' => [
                        'label' => 'スタイル',
                        'rule' => '',
                        'value' => '',
                    ],
                ];
            }
        }
        return $fieldsConfig;
    }
}
