<?php
/**
 * MicroEngine CMS
 * https://microengine.jp
 *
 * (c) 2018 MicroEngine Inc.
 */

namespace MicroEngine\Core\Middleware;

use Psr\Http\Message\ResponseInterface;
use Psr\Http\Message\ServerRequestInterface;
use Illuminate\Database\Capsule\Manager as DB;

/**
 * DBトランザクション管理ミドルウェア
 * エラーや例外発生時にトランザクションをロールバックして中途半端なDB更新を防ぐ。
 *
 * @package MicroEngine\Core\Middleware
 */
class DBTransactionMiddleware extends Middleware
{
    /**
     * @param  ServerRequestInterface $request
     * @param  ResponseInterface $response
     * @param  callable $next
     * @return ResponseInterface
     * @throws \Exception
     */
    public function __invoke(ServerRequestInterface $request, ResponseInterface $response, callable $next): ResponseInterface
    {
        // トランザクションを開始する
        DB::beginTransaction();

        try {
            // 本体処理を実行する
            $response = $next($request, $response);
        } catch (\Exception $e) {
            // 例外が発生した場合はロールバックして、中途半端にコミットしてしまわないようにする
            DB::rollBack();
            throw $e;
        }

        if ($response instanceof ResponseInterface && $response->getStatusCode() >= 400) {
            // ステータスコードが400以上であればロールバックする
            DB::rollBack();
        } else {
            // ステータスコードが400未満であればコミットする
            DB::commit();
        }
        return $response;
    }
}
