<?php
/**
 * MicroEngine CMS
 * https://microengine.jp
 *
 * (c) 2018 MicroEngine Inc.
 */

namespace MicroEngine\Core\Support;

use ZipArchive;
use MicroEngine\Core\Exception\MeException;

class Zip
{
    /**
     * @var string
     */
    protected $filePath;
    /**
     * @var ConnectionKeeper
     */
    protected $keeper;

    /**
     * @param string|null $path
     */
    public function setFilePath(string $path = null)
    {
        if ($path === null) {
            // 引数にpathが渡されなかった場合は、一時ファイル名を作成してい指定する
            $path = tempnam(sys_get_temp_dir(), 'zip');
        }
        $this->filePath = $path;
    }

    /**
     * @return string
     */
    public function getFilePath()
    {
        return $this->filePath;
    }

    /**
     * ファイルを追加する
     *
     * @param string $path 追加するファイルのパス
     * @param string $localPath アーカイブファイル内のパス
     * @throws \Exception
     */
    public function addFile(string $path, string $localPath)
    {
        // zipファイルをオープンする
        $zip = $this->open();
        // ファイルを追加する
        $zip->addFile($path, $localPath);
        // zipファイルを閉じる
        $zip->close();
    }

    /**
     * 内容を指定してファイルを追加する
     *
     * @param string $localName
     * @param string $contents
     * @throws \Exception
     */
    public function addFromString(string $localName, string $contents)
    {
        // zipファイルをオープンする
        $zip = $this->open();
        // ファイルを追加する
        $zip->addFromString($localName, $contents);
        // zipファイルを閉じる
        $zip->close();
    }

    /**
     * 複数ファイルを追加する
     * addFileとは異なり、ファイルシステム上のファイル名がそのまま使われる。
     *
     * @param array $paths 追加するファイルのパスの配列
     * @param string $basePath アーカイブファイル内のパスの起点
     * @param string $localPathPrefix アーカイブファイル内のパスのプレフィックス。例： 'system/' を指定するとアーカイブファイル直下
     * に system ディレクトリが作成されて、$basePath からの相対パス名のファイルが追加される
     * @throws \Exception
     */
    public function addFiles(array $paths, string $basePath = '/', string $localPathPrefix = '')
    {
        // zipファイルをオープンする
        $zip = $this->open();

        // ベースパスの文字数をオフセットとする
        $offset = strlen(end_slash($basePath));

        $i = 0;
        foreach ($paths as $path) {
            // $basePathからの相対パスをアーカイブファイル内のローカルパスとする
            $localPath = $localPathPrefix . substr($path, $offset);
            $zip->addFile($path, $localPath);

            if ($this->keeper) {
                // コネクションキーパーがあれば、キープを実行する
                $this->keeper->keep();
            }
            // zipファイルを閉じるときに時間がかかるので、その間のタイムアウトを防ぐためにある程度こまめにzipファイルを閉じる。
            if ($i > 1000) {
                // zipファイルを一旦閉じてファイルを書き込む
                $zip->close();
                $zip = $this->open();
                $i = 0;
            }
            $i++;
        }
        // zipファイルを閉じる
        $zip->close();
    }

    /**
     * @param ConnectionKeeper $keeper
     */
    public function setKeeper(ConnectionKeeper $keeper)
    {
        $this->keeper = $keeper;
    }

    /**
     * ZipArchive ファイルをオープンする
     *
     * @return ZipArchive
     * @throws \Exception
     */
    protected function open(): ZipArchive
    {
        if (!$this->filePath) {
            // ファイルパスが未設定の場合は一時ファイル名を設定する
            $this->setFilePath();
        }

        // アーカイブファイルを作成する
        $zip = new ZipArchive();
        if (file_exists($this->filePath)) {
            $zipOpenResult = $zip->open($this->filePath);
        } else {
            $zipOpenResult = $zip->open($this->filePath, ZipArchive::CREATE);
        }
        if ($zipOpenResult !== true) {
            throw new MeException('zipファイルのオープンに失敗しました。エラーコード：' . $zipOpenResult);
        }
        return $zip;
    }
}
