<?php
/**
 * MicroEngine CMS
 * https://microengine.jp
 *
 * (c) 2018 MicroEngine Inc.
 */

namespace MicroEngine\Install\Controllers;

use MicroEngine\Core\Support\FileSystem;
use Psr\Http\Message\ResponseInterface;
use Interop\Container\ContainerInterface;
use Psr\Http\Message\ServerRequestInterface;
use MicroEngine\Core\Controllers\Controller;
use MicroEngine\Install\Forms\UserRegisterForm;
use MicroEngine\Install\Forms\DatabaseSetupForm;

class InstallController extends Controller
{
    /**
     * @param ContainerInterface $c
     */
    public function __construct(ContainerInterface $c)
    {
        parent::__construct($c);

        // インストーラーではエラー詳細を表示する
        $this->c->get('settings')->replace(['displayErrorDetails' => 1]);
    }

    /**
     * @param  ServerRequestInterface $request
     * @param  ResponseInterface $response
     * @param  array $args
     * @return ResponseInterface
     */
    public function start(ServerRequestInterface $request, ResponseInterface $response, array $args): ResponseInterface
    {
        return $this->view->render(
            $response,
            'install/views/install/start.twig'
        );
    }

    /**
     * @param  ServerRequestInterface $request
     * @param  ResponseInterface $response
     * @param  array $args
     * @return ResponseInterface
     * @throws \MicroEngine\Core\Exception\MeException
     */
    public function eula(ServerRequestInterface $request, ResponseInterface $response, array $args): ResponseInterface
    {
        $fs = new FileSystem(system_root('microengine/Install/resources'));
        $html = $fs->get('eula_mecms.html');
        $response->getBody()->write($html);

        return $response;
    }

    /**
     * @param  ServerRequestInterface $request
     * @param  ResponseInterface $response
     * @param  array $args
     * @return ResponseInterface
     */
    public function env(ServerRequestInterface $request, ResponseInterface $response, array $args): ResponseInterface
    {
        $extensions = $this->c->installService->checkPhpExtensions();
        $paths = $this->c->installService->isWritableStorage();

        return $this->view->render(
            $response,
            'install/views/install/env.twig',
            compact('extensions', 'paths')
        );
    }

    /**
     * @param  ServerRequestInterface $request
     * @param  ResponseInterface $response
     * @param  array $args
     * @return ResponseInterface
     */
    public function createDatabase(ServerRequestInterface $request, ResponseInterface $response, array $args): ResponseInterface
    {
        $form = new DatabaseSetupForm($this->c);

        return $this->view->render(
            $response,
            'install/views/install/database.twig',
            compact('form')
        );
    }

    /**
     * @param  ServerRequestInterface $request
     * @param  ResponseInterface $response
     * @param  array $args
     * @return ResponseInterface
     * @throws \Rakit\Validation\RuleQuashException
     */
    public function storeDatabase(ServerRequestInterface $request, ResponseInterface $response, array $args): ResponseInterface
    {
        $form = new DatabaseSetupForm($this->c);
        $form->setInputValue($request->getParsedBody());

        // バリデーションチェック
        if ($form->validate() && $form->validateConnection()) {
            $config = $form->getInputValues();
            unset($config['connection']);
            $this->c->configService->update('db.php', $config);

            $this->flash->addMessage('success', 'データベース設定を保存しました。');

            return $response->withRedirect($this->c->router->pathFor('admin.install.createUser'));
        } else {
            return $this->view->render(
                $response,
                'install/views/install/database.twig',
                compact('form')
            );
        }
    }

    /**
     * @param  ServerRequestInterface $request
     * @param  ResponseInterface $response
     * @param  array $args
     * @return ResponseInterface
     */
    public function createUser(ServerRequestInterface $request, ResponseInterface $response, array $args): ResponseInterface
    {
        $form = new UserRegisterForm($this->c);

        return $this->view->render(
            $response,
            'install/views/install/user.twig',
            compact('form')
        );
    }

    /**
     * @param ServerRequestInterface $request
     * @param ResponseInterface $response
     * @param array $args
     * @return ResponseInterface
     * @throws \Rakit\Validation\RuleQuashException
     */
    public function storeUser(ServerRequestInterface $request, ResponseInterface $response, array $args): ResponseInterface
    {
        $form = new UserRegisterForm($this->c);
        $form->setInputValue($request->getParsedBody());

        // バリデーションチェック
        if ($form->validate()) {
            // DBスキーマ作成
            $this->c->installService->createDatabaseSchema();
            $data = $form->getInputValues();
            // インストール時に作成するユーザーのロールはadministratorを指定する
            $data['role'] = $this->c->userManager::SUPER_USER_ROLE;
            // ユーザー登録
            $this->c->userManager->create($data);

            // サンプルデータ登録
            if (!empty($form->get('sample'))) {
                $data = include __DIR__ . '/../resources/db/sample.php';
                $import = $this->c->import;
                $import->import($data);
            }
            $this->flash->addMessage('success', 'インストールに成功しました。');
            $this->c->logger->addNotice('インストールに成功しました。');

            return $response->withRedirect($this->c->router->pathFor('admin.auth.login'));
        } else {
            return $this->view->render(
                $response,
                'install/views/install/user.twig',
                compact('form')
            );
        }
    }
}
