<?php
/**
 * MicroEngine CMS
 * https://microengine.jp
 *
 * (c) 2018 MicroEngine Inc.
 */

namespace MicroEngine\Site\Controllers;

use MicroEngine\Cms\Models\Page;
use MicroEngine\Cms\Models\Table;
use MicroEngine\Cms\Services\Query;
use Slim\Exception\NotFoundException;
use MicroEngine\Cms\Models\Collection;
use Psr\Http\Message\ResponseInterface;
use MicroEngine\Core\Controllers\Action;
use Psr\Http\Message\ServerRequestInterface;

class MappingAction extends Action
{
    /**
     * @param  ServerRequestInterface $request
     * @param  ResponseInterface $response
     * @param  array $args
     * @return ResponseInterface
     * @throws NotFoundException
     */
    public function __invoke(ServerRequestInterface $request, ResponseInterface $response, array $args): ResponseInterface
    {
        $view = $this->c->view;

        $data = [];
        $routeName = $request->getAttribute('route')->getName();

        // 設定・ルート情報を取得
        $pagesConfig = config('pages');
        $config = [];
        $pageType = '';
        foreach ($pagesConfig as $type => $configItem) {
            $configRouteName = $configItem['options']['route_name'] ?? $type;
            if ($routeName === $configRouteName) {
                $config = $configItem;
                $pageType = $type;
                break;
            }
        }
        if (empty($config)) {
            // 設定が見つからない場合は Not Found を返す
            throw new NotFoundException($request, $response);
        }

        // 該当コンテンツの設定を取得
        $options = $config['options'];
        $options['page']['type'] = $pageType;

        if ($options['base_type'] === 'matrix') {
            $data['table'] = $this->mappingTable($options['content'], $args);
            if ($data['table'] === false) {
                // データが見つからない場合は Not Found を返す
                throw new NotFoundException($request, $response);
            }
        } elseif ($options['base_type'] === 'collection') {
            $data['collection'] = $this->mappingCollection($options['content'], $args);
            if ($data['collection'] === false) {
                // データが見つからない場合は Not Found を返す
                throw new NotFoundException($request, $response);
            }
        }

        $page = new Page();
        if (isset($options['page'])) {
            $page->fill($options['page']);
        }
        $this->c->site->pushContent($page, true);
        $data['page'] = $page;

        return $view->render(
            $response,
            $page->getTemplatePath(),
            $data
        );
    }

    /**
     * コレクションコンテンツとマッピングする
     *
     * @param string $type
     * @param array $args
     * @return bool|mixed
     * @throws \Exception
     */
    protected function mappingCollection(string $type, array $args)
    {
        $standardFields = ['id', 'type', 'name', 'display_order', 'date', 'published_at', 'created_at', 'updated_at'];

        $config = [
            'type' => $type,
            'base_type' => 'collection',
            'join' => [],
            'where' => [
                'and' => [],
            ],
        ];

        foreach ($args as $arg => $value) {
            if (in_array($arg, $standardFields)) {
                $config['where']['and'][] = [$arg, $value];
            } else {
                // カスタムフィールドの場合は join を追加
                $config['join'][] = $arg;
                $config['where']['and'][] = [$arg . '.value', $value];
            }
        }
        $query = new Query($this->c);
        $collections = $query->get($config);

        if (empty($collections) || !isset($collections[0])) {
            return false;
        }
        return $collections[0];
    }

    /**
     * テーブルコンテンツとマッピングする
     *
     * @param string $type
     * @param array $args
     * @return array|bool
     */
    protected function mappingTable(string $type, array $args)
    {
        $table = Table::type($type);

        $numArgs = count($args);
        foreach ($table as $row) {
            $matchCount = 0;
            foreach ($args as $arg => $value) {
                if ($row[$arg] == $value) {
                    $matchCount++;
                }
            }
            // パラメータ全てが一致することを確認
            if ($matchCount === $numArgs) {
                return $row;
            }
        }
        return false;
    }
}
