<?php
/**
 * public.zip 展開ツール
 * このスクリプトファイルをWebサーバーにアップロードすると、同じディレクトリ内のpublic.zipファイルを展開することができます。
 * 展開後はこのファイルとpublic.zipファイルは不要ですので、Webサーバーから削除してください。
 *
 * MicroEngine CMS
 * https://microengine.jp
 *
 * (c) 2018-2019 MicroEngine Inc.
 */

class Unzip
{
    /**
     * 展開するCMSのバージョン
     */
    const CMS_VERSION = '2.0.1';
    /**
     * アップグレード可能なCMSバージョンの下限
     */
    const UPGRADEABLE_VERSION = '2.0.0';
    /**
     * 展開するzipファイル名
     */
    const PUBLIC_ZIP = 'public.zip';
    /**
     * CMSの設定ファイル
     */
    const MECMS_CONFIG = 'mecms_config.php';

    /**
     * 展開スクリプト実行
     */
    public function run()
    {
        // 展開準備
        $this->prepare();

        // インストール済みかどうかの判定
        if (!file_exists(self::MECMS_CONFIG)) {
            // 設定ファイルがない場合は新規インストール用に展開する
            $this->forInstall();
        } else {
            // 設定ファイルがある場合はインストール済みなのでアップグレード用に展開する
            $this->forUpgrade();
        }
    }

    /**
     * 展開準備処理
     */
    protected function prepare()
    {
        // PHP設定
        ini_set('memory_limit', '256M');
        set_time_limit(300);

        // タイムゾーンの設定
        date_default_timezone_set('Asia/Tokyo');

        // 文字コード指定
        ini_set('default_charset', 'UTF-8');
        mb_internal_encoding('UTF-8');

        // PHPバージョンの確認
        if (version_compare(PHP_VERSION, '7.0.0') < 0) {
            $this->exitScreen('アプリケーションエラー', 'このツールを実行するためにはPHP7.0.0以降が必要になります。');
        }
        // zip拡張がロードされていることを確認
        if (!extension_loaded('zip')) {
            $this->exitScreen('アプリケーションエラー', 'このツールを実行するためには zip 拡張が必要になります。');
        }
        // ファイルの存在確認
        if (!file_exists(self::PUBLIC_ZIP)) {
            $this->exitScreen('アプリケーションエラー', 'public.zip ファイルが見つかりません。');
        }
    }

    /**
     * インストール用に展開する
     */
    protected function forInstall()
    {
        // zipファイルから展開するパスのリスト
        $extractPaths = array(
            'files/',
            'themes/',
            'system/',
            'index.php',
            'mecms_config.php',
            '.htaccess',
        );

        // 既存ファイル・ディレクトリ確認
        foreach ($extractPaths as $path) {
            if (file_exists($path)) {
                $this->exitScreen('アプリケーションエラー', '既存ファイル・ディレクトリが存在するため展開できません。パス：' . $this->e($path));
            }
        }

        // 書き込みパーミッション確認
        if (!is_writable('./')) {
            $this->exitScreen('アプリケーションエラー', '現在のディレクトリに書き込み権限がないため展開できません。');
        }

        // 現在ディレクトリに展開する
        $this->extract(self::PUBLIC_ZIP, '.');

        // 成功画面を表示する
        $html = <<<EOD
<p>ファイルの展開に成功しました。</p>
<main><article>
<p>展開ツールと製品パッケージファイルを削除してください。</p>
<p>次のリンクをクリックしてインストール画面に移動してください。<br>
<a href="/admin/install" target="_blank">インストール</a></p>
</article></main>
EOD;

        $this->exitScreen('展開ツール', $html);
    }

    /**
     * アップグレード用に展開する
     */
    protected function forUpgrade()
    {
        // アップグレードできるかどうか確認する。
        $mecmsConfig = require self::MECMS_CONFIG;

        // システムパスの確認
        $systemPathPrefix = null;
        foreach ($mecmsConfig['system_paths'] as $prefix) {
            if (file_exists($prefix . '/vendor/autoload.php')) {
                $systemPathPrefix = $prefix;
                break;
            }
        }
        if ($systemPathPrefix === null) {
            $this->exitScreen('アプリケーションエラー', 'autoload ファイルが見つかりません。');
        }
        // アプリケーション設定を読み込む
        $appSettings = require $systemPathPrefix . '/microengine/app_settings.php';
        if ($appSettings['version'] < self::UPGRADEABLE_VERSION) {
            $this->exitScreen('アプリケーションエラー', 'このツールでアップグレードできないバージョンがインストールされています。');
        }
        if ($appSettings['version'] >= self::CMS_VERSION) {
            $this->exitScreen('アプリケーションエラー', 'インストール済みのため、ファイルを展開できません。');
        }

        // 書き込み権限を確認するパスのリスト
        $writablePaths = array(
            '.',
            $systemPathPrefix,
            'themes',
        );

        // 作業日時
        $workDateTime = date('YmdHis');
        // 作業ディレクトリ
        $workDir = 'Upgrade_' . $workDateTime;

        // 親ディレクトリの書き込みパーミッション確認
        foreach ($writablePaths as $path) {
            if (!is_writable($path)) {
                $this->exitScreen('アプリケーションエラー', 'ディレクトリに書き込み権限がないため展開できません。パス：' . $this->e($path));
            }
        }

        // 作業ディレクトリに展開する
        $this->extract(self::PUBLIC_ZIP, $workDir);

        // 展開したファイルと書き換えるファイルのリスト
        $extractPaths = array(
            $workDir . '/index.php' => 'index.php',
            $workDir . '/system/microengine' => $systemPathPrefix . '/microengine',
            $workDir . '/system/vendor' => $systemPathPrefix . '/vendor',
            $workDir . '/system/composer.json' => $systemPathPrefix . '/composer.json',
            $workDir . '/system/composer.lock' => $systemPathPrefix . '/composer.lock',
            $workDir . '/themes/admin' => 'themes/admin',
            $workDir . '/themes/cms' => 'themes/cms',
            $workDir . '/themes/core' => 'themes/core',
            $workDir . '/themes/install' => 'themes/install',
            $workDir . '/themes/sample' => 'themes/sample',
        );

        // 展開先のファイルに作業日時を追加してリネームしてバックアップする
        foreach ($extractPaths as $path) {
            if (!file_exists($path)) {
                // ファイル・ディレクトリが存在しない場合は処理不要。sampleテーマなど不要なファイルは削除されている場合があるため
                continue;
            }
            if (!rename($path, $path . '.' . $workDateTime)) {
                $this->exitScreen('アプリケーションエラー', 'バックアップファイル（ディレクトリ）の作成に失敗しました。パス：' . $this->e($path));
            }
        }

        // 展開する
        foreach ($extractPaths as $from => $to) {
            if (!rename($from, $to)) {
                $this->exitScreen('アプリケーションエラー', 'ファイル（ディレクトリ）のリネームに失敗しました。パス：' . $this->e($from));
            }
        }

        // バックアップしたファイルを削除する
        foreach ($extractPaths as $path) {
            $backupPath = $path . '.' . $workDateTime;
            $this->deleteDirectory($backupPath);
        }

        // アーカイブを展開した作業ディレクトリを削除する
        $this->deleteDirectory($workDir);

        // 成功画面を表示する
        $html = <<<EOD
<p>ファイルの展開に成功しました。</p>
<main><article>
<p>展開ツールと製品パッケージファイルを削除してください。</p>
<p>MicroEngine CMSのスキーマバージョンが更新されている場合は、アップグレードスクリプトを実行してください。<br>
<a href="/admin/upgrade" target="_blank">アップグレードスクリプト</a><br>
<small>アップグレードスクリプトの実行が不要な場合はアプリケーションエラーが表示されます。</small><br>
<a href="/admin" target="_blank">管理画面に移動する</a>
</p>
</article></main>
EOD;

        $this->exitScreen('展開ツール', $html);
    }

    /**
     * zipファイル展開処理
     *
     * @param $filename
     * @param $to
     */
    protected function extract($filename, $to)
    {
        $zip = new ZipArchive();
        $res = $zip->open($filename);
        if ($res === TRUE) {
            // 指定ディレクトリに展開する
            $zip->extractTo($to);
            $zip->close();
        } else {
            $this->exitScreen('アプリケーションエラー', 'アーカイブファイルの展開に失敗しました。メッセージ：' . $this->e($res));
        }
    }

    /**
     * ディレクトリを再帰的に削除する
     *
     * @param $dir
     */
    protected function deleteDirectory($dir)
    {
        if (!is_dir($dir)) {
            // ファイルを渡された場合にはそのファイルだけ削除する。
            unlink($dir);
            return;
        }

        $iterator = new \RecursiveIteratorIterator(
            new \RecursiveDirectoryIterator($dir, \FilesystemIterator::SKIP_DOTS),
            \RecursiveIteratorIterator::CHILD_FIRST
        );
        foreach ($iterator as $filename => $fileInfo) {
            if ($fileInfo->isDir()) {
                rmdir($filename);
            } else {
                unlink($filename);
            }
        }
        // 最後に最初に指定したディレクトリを削除する
        rmdir($dir);
    }

    /**
     * HTMLエスケープ
     *
     * @param string $str
     * @return string
     */
    protected function e($str)
    {
        return htmlspecialchars($str, ENT_QUOTES, 'UTF-8');
    }

    /**
     * 画面をレンダリングしてスクリプトを終了する
     *
     * @param string $title
     * @param string $html
     * @return string
     */
    protected function exitScreen(string $title, string $html)
    {
        $escapedTitle = $this->e($title);

        $content = <<<EOD
<!DOCTYPE html><html lang="ja"><head><meta charset="UTF-8"><title>{$escapedTitle}</title><meta name="viewport" content="width=device-width,initial-scale=1">
<style>body{margin:0;padding:15px}h1{margin:0 0 15px;font-size:24px;color:#F70}article{background-color:#eee;padding:10px 15px}article+article{margin:20px 0 0}h2{border-bottom:1px solid #ccc;margin:0 0 10px;font-size:18px;padding:0 0 5px}pre{background-color:#f7f7f7;padding:10px;margin:0 0 5px;overflow-x:scroll}</style>
</head><body>
<header><h1>{$escapedTitle}</h1></header>
{$html}
</body></html>
EOD;

        exit($content);
    }
}

// 展開処理を実行する
$unzip = new Unzip();
$unzip->run();
